import { BaseJavaCstVisitorWithDefaults, parse } from 'java-parser';
import { pascalCase } from 'change-case';
function evalJava(code) {
    if (/^([-+]?)(\d+)L$/.test(code)) {
        code = RegExp.$1 + RegExp.$2;
    }
    try {
        return eval(code);
    }
    catch (e) {
        e.message = `Eval 执行 ${JSON.stringify(code)} 错误：${e.message}`;
        throw e;
    }
}
// 解析各种形态的comment
function formatComment(rawComment) {
    var _a, _b;
    // 识别 /** */
    let reg1 = new RegExp(/\/\*+[\/n\*\s]*([^\/n\*\s]*)[\/n\*\s]*\*+\//);
    // 识别 "//"
    let reg2 = new RegExp(/\/\/[\/n\*\s]*([^\/n\*\s]*)/);
    if (rawComment.trimStart().startsWith('//')) {
        return (_a = reg2.exec(rawComment)) === null || _a === void 0 ? void 0 : _a[1];
    }
    return (_b = reg1.exec(rawComment)) === null || _b === void 0 ? void 0 : _b[1];
}
class ExpressionValueVisitor extends BaseJavaCstVisitorWithDefaults {
    constructor() {
        super(...arguments);
        this.state = {
            prefix: null
        };
    }
    setData(raw, type = 'primitive') {
        if (this.data) {
            this.data = null;
            return;
        }
        raw = this.state.prefix != null ? this.state.prefix + raw : raw;
        let value = raw;
        if (type === 'primitive') {
            value = evalJava(raw);
        }
        this.data = {
            value,
            type,
            raw
        };
        this.state.prefix = null;
    }
    unaryExpression(ctx) {
        if (ctx.UnaryPrefixOperator) {
            this.state.prefix = ctx.UnaryPrefixOperator[0].image;
        }
        return super.unaryExpression(ctx);
    }
    integerLiteral(ctx) {
        if (ctx.DecimalLiteral) {
            this.setData(ctx.DecimalLiteral[0].image);
        }
        if (ctx.HexLiteral) {
            this.setData(ctx.HexLiteral[0].image);
        }
        if (ctx.BinaryLiteral) {
            this.setData(ctx.BinaryLiteral[0].image);
        }
        if (ctx.OctalLiteral) {
            this.setData(ctx.OctalLiteral[0].image);
        }
        return super.integerLiteral(ctx);
    }
    floatingPointLiteral(ctx) {
        if (ctx.FloatLiteral) {
            this.setData(ctx.FloatLiteral[0].image);
        }
        if (ctx.HexFloatLiteral) {
            this.setData(ctx.HexFloatLiteral[0].image);
        }
        return super.floatingPointLiteral(ctx);
    }
    booleanLiteral(ctx) {
        if (ctx.True) {
            this.setData('true');
        }
        if (ctx.False) {
            this.setData('false');
        }
    }
    literal(ctx, param) {
        if (ctx.StringLiteral) {
            this.setData(ctx.StringLiteral[0].image);
        }
        if (ctx.Null) {
            this.setData('null');
        }
        if (ctx.CharLiteral) {
            this.setData(ctx.CharLiteral[0].image);
        }
        return super.literal(ctx, param);
    }
    fqnOrRefType(ctx) {
        if (this.data) {
            this.data = null;
            return;
        }
        const firstImg = ctx.fqnOrRefTypePartFirst[0].children.fqnOrRefTypePartCommon[0].children.Identifier[0].image;
        const imgs = (ctx.fqnOrRefTypePartRest || []).map((rest) => rest.children.fqnOrRefTypePartCommon[0].children.Identifier[0].image);
        const value = [firstImg].concat(imgs);
        this.data = {
            type: 'fqnOrRefType',
            value,
            raw: value.join('.')
        };
    }
}
export function getExpressionValue(cst) {
    const v = new ExpressionValueVisitor();
    v.visit(cst);
    return v.data;
}
class Visitor extends BaseJavaCstVisitorWithDefaults {
    constructor() {
        super(...arguments);
        this.data = [];
        this.state = {
            visiting: false,
            enumClass: null,
            enums: []
        };
        // 修复第一行的comment
        this.firstEnumComment = '';
    }
    startVisit(fn) {
        this.state.visiting = true;
        if (fn) {
            const res = fn();
            this.stopVisit();
            return res;
        }
    }
    stopVisit() {
        if (this.state.visiting && this.state.enumClass) {
            this.data.push({
                enumClass: this.state.enumClass,
                enums: this.state.enums
            });
        }
        this.state.visiting = false;
        this.state.enumClass = null;
        this.state.enums = [];
    }
    isVisiting() {
        return this.state.visiting;
    }
    classDeclaration(ctx, param) {
        var _a, _b, _c, _d;
        /**
         * 和服务端经过沟通，发现这种写法不太出现，暂时先不考虑支持嵌套内部类 enum
         * @example
         * public class RoleParent {
         *  class X {
         *   public enum Type {
         *     I_DEVELOPER(1, "开发者"),
         *     I_ADMIN(2,"管理"),
         *     I_USER(3,"用户");
         *   }
         *  }
         * }
         */
        if (this.isVisiting()) {
            return;
        }
        if ((_c = (_b = (_a = ctx.classModifier) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.children) === null || _c === void 0 ? void 0 : _c.Public) {
            if ((_d = ctx.enumDeclaration) === null || _d === void 0 ? void 0 : _d.length) {
                return this.startVisit(() => super.classDeclaration(ctx, param));
            }
        }
    }
    enumDeclaration(ctx, param) {
        var _a, _b, _c, _d;
        if (this.isVisiting()) {
            this.state.enumClass = ctx.typeIdentifier[0].children.Identifier[0].image;
            // 此处修复第一行的comment
            this.firstEnumComment = (_d = (_c = (_b = (_a = ctx.enumBody) === null || _a === void 0 ? void 0 : _a[0].children) === null || _b === void 0 ? void 0 : _b.enumConstantList) === null || _c === void 0 ? void 0 : _c[0].leadingComments) === null || _d === void 0 ? void 0 : _d[0].image;
            return super.enumDeclaration(ctx, param);
        }
    }
    enumConstantList(ctx, param) {
        if (this.isVisiting()) {
            ctx.enumConstant.map((enumConstant, zIndex) => {
                var _a, _b, _c, _d, _e, _f, _g;
                let enumCtx = enumConstant.children;
                let comment = zIndex == 0 ? this.firstEnumComment : (_b = (_a = enumConstant === null || enumConstant === void 0 ? void 0 : enumConstant.leadingComments) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.image;
                comment = comment ? formatComment(comment) : undefined;
                let commentLabel = comment
                    ? {
                        type: 'primitive',
                        value: comment,
                        raw: `"${comment}"`
                    }
                    : undefined;
                let result = {
                    name: enumCtx.Identifier[0].image,
                    comment,
                    value: {
                        type: 'primitive',
                        value: enumCtx.Identifier[0].image,
                        raw: `"${enumCtx.Identifier[0].image}"`
                    },
                    label: comment ? commentLabel : undefined
                };
                const paramsLength = (_g = (_f = (_e = (_d = (_c = enumCtx.argumentList) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.children) === null || _e === void 0 ? void 0 : _e.expression) === null || _f === void 0 ? void 0 : _f.length) !== null && _g !== void 0 ? _g : 0;
                if (paramsLength == 0) {
                    if (!commentLabel) {
                        // 当参数长度是0的时候 也没有label 没有value
                        result.label = {
                            type: 'primitive',
                            value: enumCtx.Identifier[0].image,
                            raw: `"${enumCtx.Identifier[0].image}"`
                        };
                    }
                }
                else if (paramsLength == 1) {
                    // 参数长度为1的时候，第一个参数就是label(如果說這個label是英文，可以当value 并且有comment的情况共)
                    let param1 = getExpressionValue(enumCtx.argumentList[0].children.expression[0]);
                    //let isValidName = /^[^\d\W\s]+\w*$/.test(param1.value)
                    if (typeof commentLabel !== 'undefined' && param1.type == 'primitive') {
                        result.value = param1;
                        result.label = commentLabel;
                    }
                    else {
                        result.label = param1;
                    }
                }
                else if (paramsLength >= 2) {
                    // 第一个参数是VALUE，第二个参数是LABEL
                    result.value = getExpressionValue(enumCtx.argumentList[0].children.expression[0]);
                    result.label = getExpressionValue(enumCtx.argumentList[0].children.expression[1]);
                }
                this.state.enums.push(result);
            });
        }
        return super.enumConstantList(ctx, param);
    }
}
export function parseEnumCode(codeOrAst) {
    const ast = typeof codeOrAst === 'string' ? parse(codeOrAst) : codeOrAst;
    const visitor = new Visitor();
    visitor.visit(ast);
    return visitor.data;
}
/**
 * 转换当个文件
 * @param x
 * @param param1
 * @returns
 */
export function formatItemToTs(x, { transformEnumName = (s) => s } = {}) {
    const enumBodyString = x.enums
        .map((enumData) => {
        var _a;
        if (enumData.value.type === 'primitive') {
            const comment = ((_a = enumData.label) === null || _a === void 0 ? void 0 : _a.value) != null
                ? `  /**
* ${enumData.label.value}
*/`
                : '';
            return `${comment ? `${comment}\n` : ''}  ${enumData.name} = ${enumData.value.raw},`;
        }
        return '';
    })
        .filter(Boolean)
        .join('\n');
    if (!enumBodyString) {
        return '';
    }
    const name = transformEnumName(x.enumClass);
    const optionsString = x.enums
        .map((enumData) => {
        var _a, _b;
        if (enumData.value.type === 'primitive' && ((_a = enumData.label) === null || _a === void 0 ? void 0 : _a.raw)) {
            return `  { label: ${(_b = enumData.label) === null || _b === void 0 ? void 0 : _b.raw}, value: ${name}[${JSON.stringify(enumData.name)}] },`;
        }
        return '';
    })
        .filter(Boolean)
        .join('\n');
    const labelsString = x.enums
        .map((enumData) => {
        var _a;
        if (enumData.value.type === 'primitive' && ((_a = enumData.label) === null || _a === void 0 ? void 0 : _a.raw)) {
            return `  { ${enumData.name}: ${enumData.label.raw} },`;
        }
        return '';
    })
        .filter(Boolean)
        .join('\n');
    return `
export const enum ${name} {
${enumBodyString}
}
${optionsString
        ? `export const ${pascalCase(name + ' Options')} = [
${optionsString}
]
export const ${pascalCase(name + ' Labels')} = [
${labelsString}
]
`
        : ''}
`.trim();
}
export function formatToTs(data, { transformEnumName = (s) => s } = {}) {
    const option = { transformEnumName };
    return data
        .map((item) => formatItemToTs(item, option))
        .filter(Boolean)
        .join('\n');
}
